@@ -5,9 +5,11 @@
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version, or (at your opinion) under the terms
+ *  (at your option) any later version, or (at your option) under the terms
  *  of the Mozilla Public License version 2.0.
  *
+ *  ----
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -16,6 +18,12 @@
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ *  ----
+ *
+ *  This Source Code Form is subject to the terms of the Mozilla Public License,
+ *  v. 2.0. If a copy of the MPL was not distributed with this file, You can
+ *  obtain one at http://mozilla.org/MPL/2.0/.
+ *
  */
 //! This module contains the [`DoublePriorityQueue`] type and the related iterators.
 //!
@@ -23,16 +31,17 @@
 
 pub mod iterators;
 
-#[cfg(not(has_std))]
+#[cfg(not(feature = "std"))]
 use std::vec::Vec;
 
-use crate::core_iterators::{IntoIter, Iter};
-use crate::store::Store;
+use crate::core_iterators::*;
+use crate::store::{Index, Position, Store};
+use crate::TryReserveError;
 use iterators::*;
 
 use std::borrow::Borrow;
 use std::cmp::{Eq, Ord};
-#[cfg(has_std)]
+#[cfg(feature = "std")]
 use std::collections::hash_map::RandomState;
 use std::hash::{BuildHasher, Hash};
 use std::iter::{Extend, FromIterator, IntoIterator, Iterator};
@@ -79,22 +88,14 @@ use std::mem::replace;
 /// }
 /// ```
 #[derive(Clone)]
-#[cfg(has_std)]
-pub struct DoublePriorityQueue<I, P, H = RandomState>
-where
-    I: Hash + Eq,
-    P: Ord,
-{
+#[cfg(feature = "std")]
+pub struct DoublePriorityQueue<I, P, H = RandomState> {
     pub(crate) store: Store<I, P, H>,
 }
 
 #[derive(Clone)]
-#[cfg(not(has_std))]
-pub struct DoublePriorityQueue<I, P, H>
-where
-    I: Hash + Eq,
-    P: Ord,
-{
+#[cfg(not(feature = "std"))]
+pub struct DoublePriorityQueue<I, P, H> {
     pub(crate) store: Store<I, P, H>,
 }
 
@@ -118,7 +119,7 @@ where
     }
 }
 
-#[cfg(has_std)]
+#[cfg(feature = "std")]
 impl<I, P> DoublePriorityQueue<I, P>
 where
     P: Ord,
@@ -173,18 +174,106 @@ where
             store: Store::with_capacity_and_hasher(capacity, hash_builder),
         }
     }
+}
+
+impl<I, P, H> DoublePriorityQueue<I, P, H> {
+    /// Returns the number of elements the internal map can hold without
+    /// reallocating.
+    ///
+    /// This number is a lower bound; the map might be able to hold more,
+    /// but is guaranteed to be able to hold at least this many.
+    pub fn capacity(&self) -> usize {
+        self.store.capacity()
+    }
 
     /// Returns an iterator in arbitrary order over the
     /// (item, priority) elements in the queue
     pub fn iter(&self) -> Iter<I, P> {
         self.store.iter()
     }
-}
 
+    /// Clears the PriorityQueue, returning an iterator over the removed elements in arbitrary order.
+    /// If the iterator is dropped before being fully consumed, it drops the remaining elements in arbitrary order.
+    pub fn drain(&mut self) -> Drain<I, P> {
+        self.store.drain()
+    }
+
+    /// Shrinks the capacity of the internal data structures
+    /// that support this operation as much as possible.
+    pub fn shrink_to_fit(&mut self) {
+        self.store.shrink_to_fit();
+    }
+
+    /// Returns the number of elements in the priority queue.
+    #[inline]
+    pub fn len(&self) -> usize {
+        self.store.len()
+    }
+
+    /// Returns true if the priority queue contains no elements.
+    pub fn is_empty(&self) -> bool {
+        self.store.is_empty()
+    }
+
+    /// Returns the couple (item, priority) with the lowest
+    /// priority in the queue, or None if it is empty.
+    ///
+    /// Computes in **O(1)** time
+    pub fn peek_min(&self) -> Option<(&I, &P)> {
+        self.find_min().and_then(|i| {
+            self.store
+                .map
+                .get_index(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
+        })
+    }
+
+    /// Reserves capacity for at least `additional` more elements to be inserted
+    /// in the given `DoublePriorityQueue`. The collection may reserve more space to avoid
+    /// frequent reallocations. After calling `reserve`, capacity will be
+    /// greater than or equal to `self.len() + additional`. Does nothing if
+    /// capacity is already sufficient.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the new capacity overflows `usize`.
+    pub fn reserve(&mut self, additional: usize) {
+        self.store.reserve(additional);
+    }
+
+    /// Reserve capacity for `additional` more elements, without over-allocating.
+    ///
+    /// Unlike `reserve`, this does not deliberately over-allocate the entry capacity to avoid
+    /// frequent re-allocations. However, the underlying data structures may still have internal
+    /// capacity requirements, and the allocator itself may give more space than requested, so this
+    /// cannot be relied upon to be precisely minimal.
+    ///
+    /// Computes in **O(n)** time.
+    pub fn reserve_exact(&mut self, additional: usize) {
+        self.store.reserve_exact(additional);
+    }
+
+    /// Try to reserve capacity for at least `additional` more elements.
+    ///
+    /// Computes in O(n) time.
+    pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError> {
+        self.store.try_reserve(additional)
+    }
+
+    /// Try to reserve capacity for `additional` more elements, without over-allocating.
+    ///
+    /// Unlike `reserve`, this does not deliberately over-allocate the entry capacity to avoid
+    /// frequent re-allocations. However, the underlying data structures may still have internal
+    /// capacity requirements, and the allocator itself may give more space than requested, so this
+    /// cannot be relied upon to be precisely minimal.
+    ///
+    /// Computes in **O(n)** time.
+    pub fn try_reserve_exact(&mut self, additional: usize) -> Result<(), TryReserveError> {
+        self.store.try_reserve_exact(additional)
+    }
+}
 impl<I, P, H> DoublePriorityQueue<I, P, H>
 where
     P: Ord,
-    I: Hash + Eq,
 {
     /// Return an iterator in arbitrary order over the
     /// (item, priority) elements in the queue.
@@ -200,18 +289,6 @@ where
         IterMut::new(self)
     }
 
-    /// Returns the couple (item, priority) with the lowest
-    /// priority in the queue, or None if it is empty.
-    ///
-    /// Computes in **O(1)** time
-    pub fn peek_min(&self) -> Option<(&I, &P)> {
-        self.find_min().and_then(|i| {
-            self.store
-                .map
-                .get_index(unsafe { *self.store.heap.get_unchecked(i) })
-        })
-    }
-
     /// Returns the couple (item, priority) with the greatest
     /// priority in the queue, or None if it is empty.
     ///
@@ -220,67 +297,10 @@ where
         self.find_max().and_then(|i| {
             self.store
                 .map
-                .get_index(unsafe { *self.store.heap.get_unchecked(i) })
+                .get_index(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
         })
     }
 
-    /// Returns the couple (item, priority) with the greatest
-    /// priority in the queue, or None if it is empty.
-    ///
-    /// The item is a mutable reference, but it's a logic error to modify it
-    /// in a way that change the result of  `Hash` or `Eq`.
-    ///
-    /// The priority cannot be modified with a call to this function.
-    /// To modify the priority use `push`, `change_priority` or
-    /// `change_priority_by`.
-    ///
-    /// Computes in **O(1)** time
-    pub fn peek_min_mut(&mut self) -> Option<(&mut I, &P)> {
-        self.find_min()
-            .and_then(move |i| {
-                self.store
-                    .map
-                    .get_index_mut(unsafe { *self.store.heap.get_unchecked(i) })
-            })
-            .map(|(k, v)| (k, &*v))
-    }
-
-    /// Returns the couple (item, priority) with the greatest
-    /// priority in the queue, or None if it is empty.
-    ///
-    /// The item is a mutable reference, but it's a logic error to modify it
-    /// in a way that change the result of  `Hash` or `Eq`.
-    ///
-    /// The priority cannot be modified with a call to this function.
-    /// To modify the priority use `push`, `change_priority` or
-    /// `change_priority_by`.
-    ///
-    /// Computes in **O(1)** time
-    pub fn peek_max_mut(&mut self) -> Option<(&mut I, &P)> {
-        self.find_max()
-            .and_then(move |i| {
-                self.store
-                    .map
-                    .get_index_mut(unsafe { *self.store.heap.get_unchecked(i) })
-            })
-            .map(|(k, v)| (k, &*v))
-    }
-
-    /// Returns the number of elements the internal map can hold without
-    /// reallocating.
-    ///
-    /// This number is a lower bound; the map might be able to hold more,
-    /// but is guaranteed to be able to hold at least this many.
-    pub fn capacity(&self) -> usize {
-        self.store.capacity()
-    }
-
-    /// Shrinks the capacity of the internal data structures
-    /// that support this operation as much as possible.
-    pub fn shrink_to_fit(&mut self) {
-        self.store.shrink_to_fit();
-    }
-
     /// Removes the item with the lowest priority from
     /// the priority queue and returns the pair (item, priority),
     /// or None if the queue is empty.
@@ -329,16 +349,6 @@ where
         res
     }
 
-    /// Returns the number of elements in the priority queue.
-    pub fn len(&self) -> usize {
-        self.store.len()
-    }
-
-    /// Returns true if the priority queue contains no elements.
-    pub fn is_empty(&self) -> bool {
-        self.store.is_empty()
-    }
-
     /// Generates a new double ended iterator from self that
     /// will extract the elements from the one with the lowest priority
     /// to the highest one.
@@ -347,27 +357,68 @@ where
     }
 }
 
+impl<I, P, H> DoublePriorityQueue<I, P, H>
+where
+    H: BuildHasher,
+{
+    /// Returns the couple (item, priority) with the lowest
+    /// priority in the queue, or None if it is empty.
+    ///
+    /// The item is a mutable reference, but it's a logic error to modify it
+    /// in a way that change the result of  `Hash` or `Eq`.
+    ///
+    /// The priority cannot be modified with a call to this function.
+    /// To modify the priority use `push`, `change_priority` or
+    /// `change_priority_by`.
+    ///
+    /// Computes in **O(1)** time
+    pub fn peek_min_mut(&mut self) -> Option<(&mut I, &P)> {
+        use indexmap::map::MutableKeys;
+
+        self.find_min()
+            .and_then(move |i| {
+                self.store
+                    .map
+                    .get_index_mut2(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
+            })
+            .map(|(k, v)| (k, &*v))
+    }
+}
+
+impl<I, P, H> DoublePriorityQueue<I, P, H>
+where
+    P: Ord,
+    H: BuildHasher,
+{
+    /// Returns the couple (item, priority) with the greatest
+    /// priority in the queue, or None if it is empty.
+    ///
+    /// The item is a mutable reference, but it's a logic error to modify it
+    /// in a way that change the result of  `Hash` or `Eq`.
+    ///
+    /// The priority cannot be modified with a call to this function.
+    /// To modify the priority use `push`, `change_priority` or
+    /// `change_priority_by`.
+    ///
+    /// Computes in **O(1)** time
+    pub fn peek_max_mut(&mut self) -> Option<(&mut I, &P)> {
+        use indexmap::map::MutableKeys;
+        self.find_max()
+            .and_then(move |i| {
+                self.store
+                    .map
+                    .get_index_mut2(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
+            })
+            .map(|(k, v)| (k, &*v))
+    }
+}
+
 impl<I, P, H> DoublePriorityQueue<I, P, H>
 where
     P: Ord,
     I: Hash + Eq,
     H: BuildHasher,
 {
-    // reserve_exact -> IndexMap does not implement reserve_exact
-
-    /// Reserves capacity for at least `additional` more elements to be inserted
-    /// in the given `DoublePriorityQueue`. The collection may reserve more space to avoid
-    /// frequent reallocations. After calling `reserve`, capacity will be
-    /// greater than or equal to `self.len() + additional`. Does nothing if
-    /// capacity is already sufficient.
-    ///
-    /// # Panics
-    ///
-    /// Panics if the new capacity overflows `usize`.
-    pub fn reserve(&mut self, additional: usize) {
-        self.store.reserve(additional);
-    }
-
     /// Insert the item-priority pair into the queue.
     ///
     /// If an element equal to `item` was already into the queue,
@@ -377,7 +428,7 @@ where
     /// Computes in **O(log(N))** time.
     pub fn push(&mut self, item: I, priority: P) -> Option<P> {
         use indexmap::map::Entry::*;
-        let mut pos = 0;
+        let mut pos = Position(0);
         let mut oldp = None;
 
         match self.store.map.entry(item) {
@@ -396,11 +447,11 @@ where
         }
         // get a reference to the priority
         // copy the current size of the heap
-        let i = self.store.size;
+        let i = self.len();
         // add the new element in the qp vector as the last in the heap
-        self.store.qp.push(i);
-        self.store.heap.push(i);
-        self.bubble_up(i, i);
+        self.store.qp.push(Position(i));
+        self.store.heap.push(Index(i));
+        self.bubble_up(Position(i), Index(i));
         self.store.size += 1;
         None
     }
@@ -462,26 +513,34 @@ where
         I: Borrow<Q>,
         Q: Eq + Hash,
     {
-        if let Some((r, pos)) = self.store.change_priority(item, new_priority) {
-            self.up_heapify(pos);
-            Some(r)
-        } else {
-            None
-        }
+        self.store
+            .change_priority(item, new_priority)
+            .map(|(r, pos)| {
+                self.up_heapify(pos);
+                r
+            })
     }
 
     /// Change the priority of an Item using the provided function.
+    /// Return a boolean value where `true` means the item was in the queue and update was successful
+    ///
+    /// The argument `item` is only used for lookup, and is not used to overwrite the item's data
+    /// in the priority queue.
+    ///
     /// The item is found in **O(1)** thanks to the hash table.
     /// The operation is performed in **O(log(N))** time (worst case).
-    pub fn change_priority_by<Q: ?Sized, F>(&mut self, item: &Q, priority_setter: F)
+    pub fn change_priority_by<Q: ?Sized, F>(&mut self, item: &Q, priority_setter: F) -> bool
     where
         I: Borrow<Q>,
         Q: Eq + Hash,
         F: FnOnce(&mut P),
     {
-        if let Some(pos) = self.store.change_priority_by(item, priority_setter) {
-            self.up_heapify(pos);
-        }
+        self.store
+            .change_priority_by(item, priority_setter)
+            .map(|pos| {
+                self.up_heapify(pos);
+            })
+            .is_some()
     }
 
     /// Get the priority of an item, or `None`, if the item is not in the queue
@@ -531,7 +590,7 @@ where
         Q: Eq + Hash,
     {
         self.store.remove(item).map(|(item, priority, pos)| {
-            if pos < self.store.size {
+            if pos.0 < self.len() {
                 self.up_heapify(pos);
             }
 
@@ -562,23 +621,25 @@ where
     }
 }
 
-impl<I, P, H> DoublePriorityQueue<I, P, H>
-where
-    P: Ord,
-    I: Hash + Eq,
-{
+impl<I, P, H> DoublePriorityQueue<I, P, H> {
+    /// Returns the index of the min element
+    fn find_min(&self) -> Option<Position> {
+        match self.len() {
+            0 => None,
+            _ => Some(Position(0)),
+        }
+    }
 }
 
 impl<I, P, H> DoublePriorityQueue<I, P, H>
 where
     P: Ord,
-    I: Hash + Eq,
 {
     /**************************************************************************/
     /*                            internal functions                          */
 
-    fn heapify(&mut self, i: usize) {
-        if self.store.size <= 1 {
+    fn heapify(&mut self, i: Position) {
+        if self.len() <= 1 {
             return;
         }
         if level(i) % 2 == 0 {
@@ -588,204 +649,173 @@ where
         }
     }
 
-    fn heapify_min(&mut self, mut i: usize) {
-        while i <= parent(self.store.size - 1) {
+    fn heapify_min(&mut self, mut i: Position) {
+        while i <= parent(Position(self.len() - 1)) {
             let m = i;
 
+            let l = left(i);
+            let r = right(i);
             // Minimum of childs and grandchilds
-            i = *[
-                left(i),
-                right(i),
-                left(left(i)),
-                right(left(i)),
-                left(right(i)),
-                right(right(i)),
-            ]
-            .iter()
-            .filter_map(|i| self.store.heap.get(*i).map(|index| (i, index)))
-            .map(|(i, index)| {
-                self.store
-                    .map
-                    .get_index(*index)
-                    .map(|(item, priority)| (i, item, priority))
-                    .unwrap()
-            })
-            .min_by_key(|(_, _, priority)| *priority)
-            .unwrap()
-            .0;
-
-            if unsafe {
-                self.store.get_priority_from_heap_index(i)
-                    < self.store.get_priority_from_heap_index(m)
-            } {
-                self.store.swap(i, m);
-                if i > right(m) // i is a grandchild of m
-                    && unsafe {
-                        self.store.get_priority_from_heap_index(i)
-                            > self.store.get_priority_from_heap_index(parent(i))
-                    }
-                {
-                    self.store.swap(i, parent(i));
-                }
-            } else {
-                break;
-            }
-        }
-    }
-
-    fn heapify_max(&mut self, mut i: usize) {
-        while i <= parent(self.store.size - 1) {
-            let m = i;
-
-            // Minimum of childs and grandchilds
-            i = *[
-                left(i),
-                right(i),
-                left(left(i)),
-                right(left(i)),
-                left(right(i)),
-                right(right(i)),
-            ]
-            .iter()
-            .filter_map(|i| self.store.heap.get(*i).map(|index| (i, index)))
-            .map(|(i, index)| {
-                self.store
-                    .map
-                    .get_index(*index)
-                    .map(|(item, priority)| (i, item, priority))
-                    .unwrap()
-            })
-            .max_by_key(|(_, _, priority)| *priority)
-            .unwrap()
-            .0;
-
-            if unsafe {
-                self.store.get_priority_from_heap_index(i)
-                    > self.store.get_priority_from_heap_index(m)
-            } {
-                self.store.swap(i, m);
-                if i > right(m) // i is a grandchild of m
-                    && unsafe {
-                        self.store.get_priority_from_heap_index(i)
-                            < self.store.get_priority_from_heap_index(parent(i))
-                    }
-                {
-                    self.store.swap(i, parent(i));
-                }
-            } else {
-                break;
-            }
-        }
-    }
-
-    fn bubble_up(&mut self, mut position: usize, map_position: usize) -> usize {
-        if position > 0 {
-            position = if level(position) % 2 == 0 {
-                //on a min level
-                if self
-                    .store
-                    .map
-                    .get_index(unsafe { *self.store.heap.get_unchecked(parent(position)) })
-                    .unwrap()
-                    .1
-                    < self.store.map.get_index(map_position).unwrap().1
-                {
-                    // and greater then parent
-                    unsafe {
-                        *self.store.heap.get_unchecked_mut(position) =
-                            *self.store.heap.get_unchecked(parent(position));
-                        *self
-                            .store
-                            .qp
-                            .get_unchecked_mut(*self.store.heap.get_unchecked(position)) = position;
-                    }
-                    self.bubble_up_max(parent(position), map_position)
-                } else {
-                    // and less then parent
-                    self.bubble_up_min(position, map_position)
-                }
-            } else if self
-                .store
-                .map
-                .get_index(unsafe { *self.store.heap.get_unchecked(parent(position)) })
+            i = *[l, r, left(l), right(l), left(r), right(r)]
+                .iter()
+                .map_while(|i| self.store.heap.get(i.0).map(|index| (i, index)))
+                .min_by_key(|(_, index)| {
+                    self.store
+                        .map
+                        .get_index(index.0)
+                        .map(|(_, priority)| priority)
+                        .unwrap()
+                })
                 .unwrap()
-                .1
-                > self.store.map.get_index(map_position).unwrap().1
-            {
-                // on a max level and less then parent
-                unsafe {
-                    *self.store.heap.get_unchecked_mut(position) =
-                        *self.store.heap.get_unchecked(parent(position));
-                    *self
-                        .store
-                        .qp
-                        .get_unchecked_mut(*self.store.heap.get_unchecked(position)) = position;
+                .0;
+
+            if unsafe {
+                self.store.get_priority_from_position(i) < self.store.get_priority_from_position(m)
+            } {
+                self.store.swap(i, m);
+                if i > r {
+                    // i is a grandchild of m
+                    let p = parent(i);
+                    if unsafe {
+                        self.store.get_priority_from_position(i)
+                            > self.store.get_priority_from_position(p)
+                    } {
+                        self.store.swap(i, p);
+                    }
+                } else {
+                    break;
                 }
-                self.bubble_up_min(parent(position), map_position)
             } else {
+                break;
+            }
+        }
+    }
+
+    fn heapify_max(&mut self, mut i: Position) {
+        while i <= parent(Position(self.len() - 1)) {
+            let m = i;
+
+            let l = left(i);
+            let r = right(i);
+            // Minimum of childs and grandchilds
+            i = *[l, r, left(l), right(l), left(r), right(r)]
+                .iter()
+                .map_while(|i| self.store.heap.get(i.0).map(|index| (i, index)))
+                .max_by_key(|(_, index)| {
+                    self.store
+                        .map
+                        .get_index(index.0)
+                        .map(|(_, priority)| priority)
+                        .unwrap()
+                })
+                .unwrap()
+                .0;
+
+            if unsafe {
+                self.store.get_priority_from_position(i) > self.store.get_priority_from_position(m)
+            } {
+                self.store.swap(i, m);
+                if i > r {
+                    // i is a grandchild of m
+                    let p = parent(i);
+                    if unsafe {
+                        self.store.get_priority_from_position(i)
+                            < self.store.get_priority_from_position(p)
+                    } {
+                        self.store.swap(i, p);
+                    }
+                } else {
+                    break;
+                }
+            } else {
+                break;
+            }
+        }
+    }
+
+    fn bubble_up(&mut self, mut position: Position, map_position: Index) -> Position {
+        let priority = self.store.map.get_index(map_position.0).unwrap().1;
+        if position.0 > 0 {
+            let parent = parent(position);
+            let parent_priority = unsafe { self.store.get_priority_from_position(parent) };
+            let parent_index = unsafe { *self.store.heap.get_unchecked(parent.0) };
+            position = match (level(position) % 2 == 0, parent_priority < priority) {
+                // on a min level and greater then parent
+                (true, true) => {
+                    unsafe {
+                        *self.store.heap.get_unchecked_mut(position.0) = parent_index;
+                        *self.store.qp.get_unchecked_mut(parent_index.0) = position;
+                    }
+                    self.bubble_up_max(parent, map_position)
+                }
+                // on a min level and less then parent
+                (true, false) => self.bubble_up_min(position, map_position),
                 // on a max level and greater then parent
-                self.bubble_up_max(position, map_position)
+                (false, true) => self.bubble_up_max(position, map_position),
+                // on a max level and less then parent
+                (false, false) => {
+                    unsafe {
+                        *self.store.heap.get_unchecked_mut(position.0) = parent_index;
+                        *self.store.qp.get_unchecked_mut(parent_index.0) = position;
+                    }
+                    self.bubble_up_min(parent, map_position)
+                }
             }
         }
 
         unsafe {
             // put the new element into the heap and
             // update the qp translation table and the size
-            *self.store.heap.get_unchecked_mut(position) = map_position;
-            *self.store.qp.get_unchecked_mut(map_position) = position;
+            *self.store.heap.get_unchecked_mut(position.0) = map_position;
+            *self.store.qp.get_unchecked_mut(map_position.0) = position;
         }
         position
     }
 
-    fn bubble_up_min(&mut self, mut position: usize, map_position: usize) -> usize {
-        while (position > 0 && parent(position) > 0)
-            && (self
-                .store
-                .map
-                .get_index(unsafe { *self.store.heap.get_unchecked(parent(parent(position))) })
-                .unwrap()
-                .1
-                > self.store.map.get_index(map_position).unwrap().1)
-        {
+    fn bubble_up_min(&mut self, mut position: Position, map_position: Index) -> Position {
+        let priority = self.store.map.get_index(map_position.0).unwrap().1;
+        let mut grand_parent = Position(0);
+        while if position.0 > 0 && parent(position).0 > 0 {
+            grand_parent = parent(parent(position));
+            (unsafe { self.store.get_priority_from_position(grand_parent) }) > priority
+        } else {
+            false
+        } {
             unsafe {
-                *self.store.heap.get_unchecked_mut(position) =
-                    *self.store.heap.get_unchecked(parent(parent(position)));
-                *self
-                    .store
-                    .qp
-                    .get_unchecked_mut(*self.store.heap.get_unchecked(position)) = position;
+                let grand_parent_index = *self.store.heap.get_unchecked(grand_parent.0);
+                *self.store.heap.get_unchecked_mut(position.0) = grand_parent_index;
+                *self.store.qp.get_unchecked_mut(grand_parent_index.0) = position;
             }
-            position = parent(parent(position));
+            position = grand_parent;
         }
         position
     }
 
-    fn bubble_up_max(&mut self, mut position: usize, map_position: usize) -> usize {
-        while (position > 0 && parent(position) > 0)
-            && (self
-                .store
-                .map
-                .get_index(unsafe { *self.store.heap.get_unchecked(parent(parent(position))) })
-                .unwrap()
-                .1
-                < self.store.map.get_index(map_position).unwrap().1)
-        {
+    fn bubble_up_max(&mut self, mut position: Position, map_position: Index) -> Position {
+        let priority = self.store.map.get_index(map_position.0).unwrap().1;
+        let mut grand_parent = Position(0);
+        while if position.0 > 0 && parent(position).0 > 0 {
+            grand_parent = parent(parent(position));
+            (unsafe { self.store.get_priority_from_position(grand_parent) }) < priority
+        } else {
+            false
+        } {
             unsafe {
-                *self.store.heap.get_unchecked_mut(position) =
-                    *self.store.heap.get_unchecked(parent(parent(position)));
-                *self
-                    .store
-                    .qp
-                    .get_unchecked_mut(*self.store.heap.get_unchecked(position)) = position;
+                let grand_parent_index = *self.store.heap.get_unchecked(grand_parent.0);
+                *self.store.heap.get_unchecked_mut(position.0) = grand_parent_index;
+                *self.store.qp.get_unchecked_mut(grand_parent_index.0) = position;
             }
-            position = parent(parent(position));
+            position = grand_parent;
         }
         position
     }
 
-    fn up_heapify(&mut self, i: usize) {
-        let tmp = unsafe { *self.store.heap.get_unchecked(i) };
+    fn up_heapify(&mut self, i: Position) {
+        let tmp = unsafe { *self.store.heap.get_unchecked(i.0) };
         let pos = self.bubble_up(i, tmp);
+        if i != pos {
+            self.heapify(i)
+        }
         self.heapify(pos);
     }
 
@@ -794,36 +824,28 @@ where
     ///
     /// Computes in **O(N)**
     pub(crate) fn heap_build(&mut self) {
-        if self.store.size == 0 {
+        if self.is_empty() {
             return;
         }
-        for i in (0..=parent(self.store.size)).rev() {
-            self.heapify(i);
+        for i in (0..=parent(Position(self.len())).0).rev() {
+            self.heapify(Position(i));
         }
     }
 
     /// Returns the index of the max element
-    fn find_max(&self) -> Option<usize> {
-        match self.store.size {
+    fn find_max(&self) -> Option<Position> {
+        match self.len() {
             0 => None,
-            1 => Some(0),
-            2 => Some(1),
+            1 => Some(Position(0)),
+            2 => Some(Position(1)),
             _ => Some(
-                *[1, 2]
+                *[Position(1), Position(2)]
                     .iter()
-                    .max_by_key(|i| unsafe { self.store.get_priority_from_heap_index(**i) })
+                    .max_by_key(|i| unsafe { self.store.get_priority_from_position(**i) })
                     .unwrap(),
             ),
         }
     }
-
-    /// Returns the index of the min element
-    fn find_min(&self) -> Option<usize> {
-        match self.store.size {
-            0 => None,
-            _ => Some(0),
-        }
-    }
 }
 
 //FIXME: fails when the vector contains repeated items
@@ -908,6 +930,7 @@ impl<'a, I, P, H> IntoIterator for &'a mut DoublePriorityQueue<I, P, H>
 where
     I: Hash + Eq,
     P: Ord,
+    H: BuildHasher,
 {
     type Item = (&'a mut I, &'a mut P);
     type IntoIter = IterMut<'a, I, P, H>;
@@ -927,10 +950,10 @@ where
         let (min, max) = iter.size_hint();
         let rebuild = if let Some(max) = max {
             self.reserve(max);
-            better_to_rebuild(self.store.size, max)
+            better_to_rebuild(self.len(), max)
         } else if min != 0 {
             self.reserve(min);
-            better_to_rebuild(self.store.size, min)
+            better_to_rebuild(self.len(), min)
         } else {
             false
         };
@@ -975,27 +998,30 @@ where
 }
 
 /// Compute the index of the left child of an item from its index
-fn left(i: usize) -> usize {
-    (i * 2) + 1
+#[inline(always)]
+const fn left(i: Position) -> Position {
+    Position((i.0 * 2) + 1)
 }
 /// Compute the index of the right child of an item from its index
-fn right(i: usize) -> usize {
-    (i * 2) + 2
+#[inline(always)]
+const fn right(i: Position) -> Position {
+    Position((i.0 * 2) + 2)
 }
 /// Compute the index of the parent element in the heap from its index
-fn parent(i: usize) -> usize {
-    (i - 1) / 2
+#[inline(always)]
+const fn parent(i: Position) -> Position {
+    Position((i.0 - 1) / 2)
 }
 
 // Compute the level of a node from its index
-fn level(i: usize) -> usize {
-    log2_fast(i + 1)
+#[inline(always)]
+const fn level(i: Position) -> usize {
+    log2_fast(i.0 + 1)
 }
 
-fn log2_fast(x: usize) -> usize {
-    use std::mem::size_of;
-
-    8 * size_of::<usize>() - (x.leading_zeros() as usize) - 1
+#[inline(always)]
+const fn log2_fast(x: usize) -> usize {
+    (usize::BITS - x.leading_zeros() - 1) as usize
 }
 
 // `rebuild` takes O(len1 + len2) operations
@@ -1014,6 +1040,7 @@ fn better_to_rebuild(len1: usize, len2: usize) -> bool {
 }
 
 #[cfg(feature = "serde")]
+#[cfg_attr(docsrs, doc(cfg(feature = "serde")))]
 mod serde {
     use std::cmp::{Eq, Ord};
     use std::hash::{BuildHasher, Hash};
