@@ -5,9 +5,11 @@
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version, or (at your opinion) under the terms
+ *  (at your option) any later version, or (at your option) under the terms
  *  of the Mozilla Public License version 2.0.
  *
+ *  ----
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -16,6 +18,12 @@
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ *  ----
+ *
+ *  This Source Code Form is subject to the terms of the Mozilla Public License,
+ *  v. 2.0. If a copy of the MPL was not distributed with this file, You can
+ *  obtain one at http://mozilla.org/MPL/2.0/.
+ *
  */
 
 //! This module contains the [`PriorityQueue`] type and the related iterators.
@@ -24,16 +32,17 @@
 
 pub mod iterators;
 
-#[cfg(not(has_std))]
+#[cfg(not(feature = "std"))]
 use std::vec::Vec;
 
-use crate::core_iterators::{IntoIter, Iter};
-use crate::store::Store;
+use crate::core_iterators::*;
+use crate::store::{Index, Position, Store};
+use crate::TryReserveError;
 use iterators::*;
 
 use std::borrow::Borrow;
 use std::cmp::{Eq, Ord};
-#[cfg(has_std)]
+#[cfg(feature = "std")]
 use std::collections::hash_map::RandomState;
 use std::hash::{BuildHasher, Hash};
 use std::iter::{Extend, FromIterator, IntoIterator, Iterator};
@@ -70,22 +79,14 @@ use std::mem::replace;
 /// }
 /// ```
 #[derive(Clone, Debug)]
-#[cfg(has_std)]
-pub struct PriorityQueue<I, P, H = RandomState>
-where
-    I: Hash + Eq,
-    P: Ord,
-{
+#[cfg(feature = "std")]
+pub struct PriorityQueue<I, P, H = RandomState> {
     pub(crate) store: Store<I, P, H>,
 }
 
 #[derive(Clone, Debug)]
-#[cfg(not(has_std))]
-pub struct PriorityQueue<I, P, H>
-where
-    I: Hash + Eq,
-    P: Ord,
-{
+#[cfg(not(feature = "std"))]
+pub struct PriorityQueue<I, P, H> {
     pub(crate) store: Store<I, P, H>,
 }
 
@@ -109,7 +110,7 @@ where
     }
 }
 
-#[cfg(has_std)]
+#[cfg(feature = "std")]
 impl<I, P> PriorityQueue<I, P>
 where
     P: Ord,
@@ -164,18 +165,116 @@ where
             store: Store::with_capacity_and_hasher(capacity, hash_builder),
         }
     }
+}
 
+impl<I, P, H> PriorityQueue<I, P, H> {
     /// Returns an iterator in arbitrary order over the
     /// (item, priority) elements in the queue
     pub fn iter(&self) -> Iter<I, P> {
         self.store.iter()
     }
+
+    /// Shrinks the capacity of the internal data structures
+    /// that support this operation as much as possible.
+    pub fn shrink_to_fit(&mut self) {
+        self.store.shrink_to_fit();
+    }
+
+    /// Clears the PriorityQueue, returning an iterator over the removed elements in arbitrary order.
+    /// If the iterator is dropped before being fully consumed, it drops the remaining elements in arbitrary order.
+    pub fn drain(&mut self) -> Drain<I, P> {
+        self.store.drain()
+    }
+
+    /// Returns the couple (item, priority) with the greatest
+    /// priority in the queue, or None if it is empty.
+    ///
+    /// Computes in **O(1)** time
+    pub fn peek(&self) -> Option<(&I, &P)> {
+        self.store
+            .heap
+            .first()
+            .and_then(|index| self.store.map.get_index(index.0))
+    }
+
+    /// Returns the number of elements the internal map can hold without
+    /// reallocating.
+    ///
+    /// This number is a lower bound; the map might be able to hold more,
+    /// but is guaranteed to be able to hold at least this many.
+    pub fn capacity(&self) -> usize {
+        self.store.capacity()
+    }
+
+    /// Returns the number of elements in the priority queue.
+    #[inline]
+    pub fn len(&self) -> usize {
+        self.store.len()
+    }
+
+    /// Returns true if the priority queue contains no elements.
+    pub fn is_empty(&self) -> bool {
+        self.store.is_empty()
+    }
+
+    /// Returns the items not ordered
+    pub fn into_vec(self) -> Vec<I> {
+        self.store.into_vec()
+    }
+
+    /// Drops all items from the priority queue
+    pub fn clear(&mut self) {
+        self.store.clear();
+    }
+
+    /// Reserves capacity for at least `additional` more elements to be inserted
+    /// in the given `PriorityQueue`. The collection may reserve more space to avoid
+    /// frequent reallocations. After calling `reserve`, capacity will be
+    /// greater than or equal to `self.len() + additional`. Does nothing if
+    /// capacity is already sufficient.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the new capacity overflows `usize`.
+    pub fn reserve(&mut self, additional: usize) {
+        self.store.reserve(additional);
+    }
+
+    /// Reserve capacity for `additional` more elements, without over-allocating.
+    ///
+    /// Unlike `reserve`, this does not deliberately over-allocate the entry capacity to avoid
+    /// frequent re-allocations. However, the underlying data structures may still have internal
+    /// capacity requirements, and the allocator itself may give more space than requested, so this
+    /// cannot be relied upon to be precisely minimal.
+    ///
+    /// Computes in **O(n)** time.
+    pub fn reserve_exact(&mut self, additional: usize) {
+        self.store.reserve_exact(additional);
+    }
+
+    /// Try to reserve capacity for at least `additional` more elements.
+    ///
+    /// Computes in O(n) time.
+    pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError> {
+        self.store.try_reserve(additional)
+    }
+
+    /// Try to reserve capacity for `additional` more elements, without over-allocating.
+    ///
+    /// Unlike `reserve`, this does not deliberately over-allocate the entry capacity to avoid
+    /// frequent re-allocations. However, the underlying data structures may still have internal
+    /// capacity requirements, and the allocator itself may give more space than requested, so this
+    /// cannot be relied upon to be precisely minimal.
+    ///
+    /// Computes in **O(n)** time.
+    pub fn try_reserve_exact(&mut self, additional: usize) -> Result<(), TryReserveError> {
+        self.store.try_reserve_exact(additional)
+    }
 }
 
 impl<I, P, H> PriorityQueue<I, P, H>
 where
     P: Ord,
-    I: Hash + Eq,
 {
     /// Returns an iterator in arbitrary order over the
     /// (item, priority) elements in the queue.
@@ -191,65 +290,16 @@ where
         IterMut::new(self)
     }
 
-    /// Returns the couple (item, priority) with the greatest
-    /// priority in the queue, or None if it is empty.
-    ///
-    /// Computes in **O(1)** time
-    pub fn peek(&self) -> Option<(&I, &P)> {
-        if self.store.size == 0 {
-            return None;
-        }
-        self.store
-            .map
-            .get_index(unsafe { *self.store.heap.get_unchecked(0) })
-    }
-
-    /// Returns the couple (item, priority) with the greatest
-    /// priority in the queue, or None if it is empty.
-    ///
-    /// The item is a mutable reference, but it's a logic error to modify it
-    /// in a way that change the result of  `Hash` or `Eq`.
-    ///
-    /// The priority cannot be modified with a call to this function.
-    /// To modify the priority use `push`, `change_priority` or
-    /// `change_priority_by`.
-    ///
-    /// Computes in **O(1)** time
-    pub fn peek_mut(&mut self) -> Option<(&mut I, &P)> {
-        if self.store.size == 0 {
-            return None;
-        }
-        self.store
-            .map
-            .get_index_mut(unsafe { *self.store.heap.get_unchecked(0) })
-            .map(|(k, v)| (k, &*v))
-    }
-
-    /// Returns the number of elements the internal map can hold without
-    /// reallocating.
-    ///
-    /// This number is a lower bound; the map might be able to hold more,
-    /// but is guaranteed to be able to hold at least this many.
-    pub fn capacity(&self) -> usize {
-        self.store.capacity()
-    }
-
-    /// Shrinks the capacity of the internal data structures
-    /// that support this operation as much as possible.
-    pub fn shrink_to_fit(&mut self) {
-        self.store.shrink_to_fit();
-    }
-
     /// Removes the item with the greatest priority from
     /// the priority queue and returns the pair (item, priority),
     /// or None if the queue is empty.
     pub fn pop(&mut self) -> Option<(I, P)> {
-        match self.store.size {
+        match self.len() {
             0 => None,
-            1 => self.store.swap_remove(0),
+            1 => self.store.swap_remove(Position(0)),
             _ => {
-                let r = self.store.swap_remove(0);
-                self.heapify(0);
+                let r = self.store.swap_remove(Position(0));
+                self.heapify(Position(0));
                 r
             }
         }
@@ -266,16 +316,6 @@ where
         res
     }
 
-    /// Returns the number of elements in the priority queue.
-    pub fn len(&self) -> usize {
-        self.store.len()
-    }
-
-    /// Returns true if the priority queue contains no elements.
-    pub fn is_empty(&self) -> bool {
-        self.store.is_empty()
-    }
-
     /// Generates a new iterator from self that
     /// will extract the elements from the one with the highest priority
     /// to the lowest one.
@@ -290,21 +330,6 @@ where
     I: Hash + Eq,
     H: BuildHasher,
 {
-    // reserve_exact -> IndexMap does not implement reserve_exact
-
-    /// Reserves capacity for at least `additional` more elements to be inserted
-    /// in the given `PriorityQueue`. The collection may reserve more space to avoid
-    /// frequent reallocations. After calling `reserve`, capacity will be
-    /// greater than or equal to `self.len() + additional`. Does nothing if
-    /// capacity is already sufficient.
-    ///
-    /// # Panics
-    ///
-    /// Panics if the new capacity overflows `usize`.
-    pub fn reserve(&mut self, additional: usize) {
-        self.store.reserve(additional);
-    }
-
     /// Insert the item-priority pair into the queue.
     ///
     /// If an element equal to `item` was already into the queue,
@@ -314,7 +339,7 @@ where
     /// Computes in **O(log(N))** time.
     pub fn push(&mut self, item: I, priority: P) -> Option<P> {
         use indexmap::map::Entry::*;
-        let mut pos = 0;
+        let mut pos = Position(0);
         let mut oldp = None;
 
         match self.store.map.entry(item) {
@@ -332,11 +357,11 @@ where
             return oldp;
         }
         // copy the current size of the heap
-        let i = self.store.size;
+        let i = self.len();
         // add the new element in the qp vector as the last in the heap
-        self.store.qp.push(i);
-        self.store.heap.push(i);
-        self.bubble_up(i, i);
+        self.store.qp.push(Position(i));
+        self.store.heap.push(Index(i));
+        self.bubble_up(Position(i), Index(i));
         self.store.size += 1;
         None
     }
@@ -398,26 +423,34 @@ where
         I: Borrow<Q>,
         Q: Eq + Hash,
     {
-        if let Some((r, pos)) = self.store.change_priority(item, new_priority) {
-            self.up_heapify(pos);
-            Some(r)
-        } else {
-            None
-        }
+        self.store
+            .change_priority(item, new_priority)
+            .map(|(r, pos)| {
+                self.up_heapify(pos);
+                r
+            })
     }
 
     /// Change the priority of an Item using the provided function.
+    /// Return a boolean value where `true` means the item was in the queue and update was successful
+    ///
+    /// The argument `item` is only used for lookup, and is not used to overwrite the item's data
+    /// in the priority queue.
+    ///
     /// The item is found in **O(1)** thanks to the hash table.
     /// The operation is performed in **O(log(N))** time (worst case).
-    pub fn change_priority_by<Q: ?Sized, F>(&mut self, item: &Q, priority_setter: F)
+    pub fn change_priority_by<Q: ?Sized, F>(&mut self, item: &Q, priority_setter: F) -> bool
     where
         I: Borrow<Q>,
         Q: Eq + Hash,
         F: FnOnce(&mut P),
     {
-        if let Some(pos) = self.store.change_priority_by(item, priority_setter) {
-            self.up_heapify(pos);
-        }
+        self.store
+            .change_priority_by(item, priority_setter)
+            .map(|pos| {
+                self.up_heapify(pos);
+            })
+            .is_some()
     }
 
     /// Get the priority of an item, or `None`, if the item is not in the queue
@@ -456,6 +489,28 @@ where
         self.store.get_mut(item)
     }
 
+    /// Returns the couple (item, priority) with the greatest
+    /// priority in the queue, or None if it is empty.
+    ///
+    /// The item is a mutable reference, but it's a logic error to modify it
+    /// in a way that change the result of  `Hash` or `Eq`.
+    ///
+    /// The priority cannot be modified with a call to this function.
+    /// To modify the priority use `push`, `change_priority` or
+    /// `change_priority_by`.
+    ///
+    /// Computes in **O(1)** time
+    pub fn peek_mut(&mut self) -> Option<(&mut I, &P)> {
+        use indexmap::map::MutableKeys;
+        if self.store.size == 0 {
+            return None;
+        }
+        self.store
+            .map
+            .get_index_mut2(unsafe { self.store.heap.get_unchecked(0) }.0)
+            .map(|(k, v)| (k, &*v))
+    }
+
     /// Remove an arbitrary element from the priority queue.
     /// Returns the (item, priority) couple or None if the item
     /// is not found in the queue.
@@ -467,7 +522,7 @@ where
         Q: Eq + Hash,
     {
         self.store.remove(item).map(|(item, priority, pos)| {
-            if pos < self.store.size {
+            if pos.0 < self.len() {
                 self.up_heapify(pos);
             }
 
@@ -475,16 +530,6 @@ where
         })
     }
 
-    /// Returns the items not ordered
-    pub fn into_vec(self) -> Vec<I> {
-        self.store.into_vec()
-    }
-
-    /// Drops all items from the priority queue
-    pub fn clear(&mut self) {
-        self.store.clear();
-    }
-
     /// Move all items of the `other` queue to `self`
     /// ignoring the items Eq to elements already in `self`
     /// At the end, `other` will be empty.
@@ -508,7 +553,6 @@ where
 impl<I, P, H> PriorityQueue<I, P, H>
 where
     P: Ord,
-    I: Hash + Eq,
 {
     /**************************************************************************/
     /*                            internal functions                          */
@@ -516,72 +560,71 @@ where
     /// Internal function that restores the functional property of the sub-heap rooted in `i`
     ///
     /// Computes in **O(log(N))** time
-    fn heapify(&mut self, mut i: usize) {
-        let (mut l, mut r) = (left(i), right(i));
-        let mut largest = if l < self.store.size
-            && unsafe {
-                self.store.get_priority_from_heap_index(l)
-                    > self.store.get_priority_from_heap_index(i)
-            } {
-            l
-        } else {
-            i
-        };
+    fn heapify(&mut self, mut i: Position) {
+        if self.len() <= 1 {
+            return;
+        }
 
-        if r < self.store.size
-            && unsafe {
-                self.store.get_priority_from_heap_index(r)
-                    > self.store.get_priority_from_heap_index(largest)
+        let (mut l, mut r) = (left(i), right(i));
+        let mut largest = i;
+
+        let mut largestp = unsafe { self.store.get_priority_from_position(i) };
+        if l.0 < self.len() {
+            let childp = unsafe { self.store.get_priority_from_position(l) };
+            if childp > largestp {
+                largest = l;
+                largestp = childp;
+            }
+
+            if r.0 < self.len() && unsafe { self.store.get_priority_from_position(r) } > largestp {
+                largest = r;
             }
-        {
-            largest = r;
         }
 
         while largest != i {
             self.store.swap(i, largest);
 
             i = largest;
+            let mut largestp = unsafe { self.store.get_priority_from_position(i) };
             l = left(i);
-            r = right(i);
-            if l < self.store.size
-                && unsafe {
-                    self.store.get_priority_from_heap_index(l)
-                        > self.store.get_priority_from_heap_index(i)
+            if l.0 < self.len() {
+                let childp = unsafe { self.store.get_priority_from_position(l) };
+                if childp > largestp {
+                    largest = l;
+                    largestp = childp;
                 }
-            {
-                largest = l;
-            } else {
-                largest = i;
-            }
-            if r < self.store.size
-                && unsafe {
-                    self.store.get_priority_from_heap_index(r)
-                        > self.store.get_priority_from_heap_index(largest)
+
+                r = right(i);
+                if r.0 < self.len()
+                    && unsafe { self.store.get_priority_from_position(r) } > largestp
+                {
+                    largest = r;
                 }
-            {
-                largest = r;
             }
         }
     }
 
     /// from the leaf go up to root or until an element with priority greater
     /// than the new element is found
-    fn bubble_up(&mut self, mut position: usize, map_position: usize) -> usize {
-        unsafe {
-            while (position > 0)
-                && (self.store.get_priority_from_heap_index(parent(position))
-                    < self.store.map.get_index(map_position).unwrap().1)
-            {
-                *self.store.heap.get_unchecked_mut(position) =
-                    *self.store.heap.get_unchecked(parent(position));
-                *self
-                    .store
-                    .qp
-                    .get_unchecked_mut(*self.store.heap.get_unchecked(position)) = position;
-                position = parent(position);
+    fn bubble_up(&mut self, mut position: Position, map_position: Index) -> Position {
+        let priority = self.store.map.get_index(map_position.0).unwrap().1;
+        let mut parent_position = Position(0);
+        while if position.0 > 0 {
+            parent_position = parent(position);
+            (unsafe { self.store.get_priority_from_position(parent_position) }) < priority
+        } else {
+            false
+        } {
+            unsafe {
+                let parent_index = *self.store.heap.get_unchecked(parent_position.0);
+                *self.store.heap.get_unchecked_mut(position.0) = parent_index;
+                *self.store.qp.get_unchecked_mut(parent_index.0) = position;
             }
-            *self.store.heap.get_unchecked_mut(position) = map_position;
-            *self.store.qp.get_unchecked_mut(map_position) = position;
+            position = parent_position;
+        }
+        unsafe {
+            *self.store.heap.get_unchecked_mut(position.0) = map_position;
+            *self.store.qp.get_unchecked_mut(map_position.0) = position;
         }
         position
     }
@@ -590,8 +633,8 @@ where
     /// and restores the functional property
     ///
     /// Computes in **O(log(N))**
-    fn up_heapify(&mut self, i: usize) {
-        let tmp = unsafe { *self.store.heap.get_unchecked(i) };
+    fn up_heapify(&mut self, i: Position) {
+        let tmp = unsafe { *self.store.heap.get_unchecked(i.0) };
         let pos = self.bubble_up(i, tmp);
         self.heapify(pos)
     }
@@ -601,11 +644,11 @@ where
     ///
     /// Computes in **O(N)**
     pub(crate) fn heap_build(&mut self) {
-        if self.store.size == 0 {
+        if self.is_empty() {
             return;
         }
-        for i in (0..=parent(self.store.size)).rev() {
-            self.heapify(i);
+        for i in (0..=parent(Position(self.len())).0).rev() {
+            self.heapify(Position(i));
         }
     }
 }
@@ -692,6 +735,7 @@ impl<'a, I, P, H> IntoIterator for &'a mut PriorityQueue<I, P, H>
 where
     I: Hash + Eq,
     P: Ord,
+    H: BuildHasher,
 {
     type Item = (&'a mut I, &'a mut P);
     type IntoIter = IterMut<'a, I, P, H>;
@@ -711,10 +755,10 @@ where
         let (min, max) = iter.size_hint();
         let rebuild = if let Some(max) = max {
             self.reserve(max);
-            better_to_rebuild(self.store.size, max)
+            better_to_rebuild(self.len(), max)
         } else if min != 0 {
             self.reserve(min);
-            better_to_rebuild(self.store.size, min)
+            better_to_rebuild(self.len(), min)
         } else {
             false
         };
@@ -747,21 +791,24 @@ where
 }
 
 /// Compute the index of the left child of an item from its index
-fn left(i: usize) -> usize {
-    (i * 2) + 1
+#[inline(always)]
+const fn left(i: Position) -> Position {
+    Position((i.0 * 2) + 1)
 }
 /// Compute the index of the right child of an item from its index
-fn right(i: usize) -> usize {
-    (i * 2) + 2
+#[inline(always)]
+const fn right(i: Position) -> Position {
+    Position((i.0 * 2) + 2)
 }
 /// Compute the index of the parent element in the heap from its index
-fn parent(i: usize) -> usize {
-    (i - 1) / 2
+#[inline(always)]
+const fn parent(i: Position) -> Position {
+    Position((i.0 - 1) / 2)
 }
 
-fn log2_fast(x: usize) -> usize {
-    use std::mem::size_of;
-    8 * size_of::<usize>() - (x.leading_zeros() as usize) - 1
+#[inline(always)]
+const fn log2_fast(x: usize) -> usize {
+    (usize::BITS - x.leading_zeros() - 1) as usize
 }
 
 // `rebuild` takes O(len1 + len2) operations
@@ -780,6 +827,7 @@ fn better_to_rebuild(len1: usize, len2: usize) -> bool {
 }
 
 #[cfg(feature = "serde")]
+#[cfg_attr(docsrs, doc(cfg(feature = "serde")))]
 mod serde {
     use std::cmp::{Eq, Ord};
     use std::hash::{BuildHasher, Hash};
