@@ -5,9 +5,11 @@
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version, or (at your opinion) under the terms
+ *  (at your option) any later version, or (at your option) under the terms
  *  of the Mozilla Public License version 2.0.
  *
+ *  ----
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -16,6 +18,12 @@
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ *  ----
+ *
+ *  This Source Code Form is subject to the terms of the Mozilla Public License,
+ *  v. 2.0. If a copy of the MPL was not distributed with this file, You can
+ *  obtain one at http://mozilla.org/MPL/2.0/.
+ *
  */
 
 #[cfg(test)]
@@ -82,8 +90,7 @@ mod pqueue_tests {
 
         let processor_priority = |pq: &PriorityQueue<&str, i32>| {
             *pq.iter()
-                .filter_map(|(i, p)| if *i == "Processor" { Some(p) } else { None })
-                .next()
+                .find_map(|(i, p)| if *i == "Processor" { Some(p) } else { None })
                 .unwrap()
         };
 
@@ -228,7 +235,7 @@ mod pqueue_tests {
         use std::iter::FromIterator;
 
         let v = vec![("a", 1), ("b", 2), ("f", 7)];
-        let mut pq: PriorityQueue<_, _> = PriorityQueue::from_iter(v.into_iter());
+        let mut pq: PriorityQueue<_, _> = PriorityQueue::from_iter(v);
         assert_eq!(pq.pop(), Some(("f", 7)));
         assert_eq!(pq.len(), 2);
     }
@@ -247,9 +254,10 @@ mod pqueue_tests {
         use std::iter::FromIterator;
 
         let v = vec![("a", 1), ("b", 2), ("f", 7), ("g", 6), ("h", 5)];
-        let mut pq: PriorityQueue<_, _> = PriorityQueue::from_iter(v.into_iter());
+        let mut pq: PriorityQueue<_, _> = PriorityQueue::from_iter(v);
 
-        pq.change_priority_by("b", |b| *b += 8);
+        assert!(!pq.change_priority_by("z", |z| *z += 8));
+        assert!(pq.change_priority_by("b", |b| *b += 8));
         assert_eq!(pq.into_sorted_vec().as_slice(), &["b", "f", "g", "h", "a"]);
     }
 
@@ -277,7 +285,7 @@ mod pqueue_tests {
         type Pq<I, P> = PriorityQueue<I, P>;
 
         let v = vec![("a", 1), ("b", 2), ("f", 7), ("g", 6), ("h", 5)];
-        let mut pq = Pq::from_iter(v.into_iter());
+        let mut pq = Pq::from_iter(v);
 
         pq.remove(&"b").unwrap();
         pq.push("b", 2);
@@ -319,6 +327,67 @@ mod pqueue_tests {
         }
     }
 
+    #[test]
+    fn drain() {
+        use std::collections::hash_map::RandomState;
+        let mut queue = PriorityQueue::<i32, i32, RandomState>::default();
+
+        for i in 0..7 {
+            queue.push(i, i);
+        }
+
+        let previous_capacity = queue.capacity();
+        queue.drain();
+
+        assert_eq!(queue.len(), 0);
+        assert_eq!(queue.capacity(), previous_capacity);
+        assert_eq!(queue.pop(), None);
+    }
+
+    #[test]
+    fn reserve() {
+        use std::collections::hash_map::RandomState;
+        let mut queue = PriorityQueue::<i32, i32, RandomState>::default();
+
+        queue.reserve(100);
+
+        assert_eq!(queue.len(), 0);
+        assert!(queue.capacity() >= 100);
+    }
+
+    #[test]
+    fn reserve_exact() {
+        use std::collections::hash_map::RandomState;
+        let mut queue = PriorityQueue::<i32, i32, RandomState>::default();
+
+        queue.reserve_exact(100);
+
+        assert_eq!(queue.len(), 0);
+        assert_eq!(queue.capacity(), 100);
+    }
+
+    #[test]
+    fn try_reserve() {
+        use std::collections::hash_map::RandomState;
+        let mut queue = PriorityQueue::<i32, i32, RandomState>::default();
+
+        queue.try_reserve(100).unwrap();
+
+        assert_eq!(queue.len(), 0);
+        assert!(queue.capacity() >= 100);
+    }
+
+    #[test]
+    fn try_reserve_exact() {
+        use std::collections::hash_map::RandomState;
+        let mut queue = PriorityQueue::<i32, i32, RandomState>::default();
+
+        queue.try_reserve_exact(100).unwrap();
+
+        assert_eq!(queue.len(), 0);
+        assert_eq!(queue.capacity(), 100);
+    }
+
     #[test]
     fn extend() {
         let mut pq = PriorityQueue::new();
